/*************************************************************************************

       <<  SpAD ver 0.1 >>  x Linux
       --------------------------------------
	by Logiman - http://www.logiman.com
       --------------------------------------

- Speaker Alarm Dir watches a selected directory and plays a custom PC speaker
  sound when a file is added or removed.

- Monitorizza il numero di file in una directory emettendo un allarme 
  sonoro quando viene rimosso o aggiunto un file.

Compiling:
---------------------
	#> gcc spad.c -o spad

Usage:
--------
	#> spad "directory" "pause (in second)" [+/-]

	Il parametro opzionle "+" o "-" serve ad indicare se deve suonare
	solo quando viene aggiunto o solo quando viene rimosso un file.
        Se non viene indicato nulla "l'allarme scatta" in entrambi i casi.

	es:
	
	#> spad /var/mydir 5
        -------------------------------------------------------------------
		plays a custom PC speaker sound when a file is added or removed.

		Controlla la directory "/var/mydir" ogni 5 secondi e 
		suona se cambia il numero di files (aumenta o diminuisce)
	
	#> spad /var/mydir 3 -
	-------------------------------------------------------------------
		plays a custom PC speaker sound only when a file is removed.

		Controlla la directory "/var/mydir" ogni 3 secondi e 
		suona solo se viene eliminato uno o piu' files

	#> spad /home/user/Maildir/new 1 + &
	-------------------------------------------------------------------
		plays a custom PC speaker sound only when a file is added.
		(work in background)

		Lavora in background controllando la directory 
		"/home/user/Maildir/new" ogni secondo e suona solo se viene
		aggiunto uno o piu' file

Thanks:
---------
    Stas (http://sysd.org/) per il codice d'esempio "demoniac.c"
    Eugeni Dodonov (http://www.gsdr.dc.ufscar.br/~eugeni/) per LibPCSpeaker

*************************************************************************************/

// speaker
#include <asm/io.h>
#include <signal.h>
#include <stdio.h>
// directory 
#include <dirent.h>        
#include <unistd.h> 

#define OUTPORT(p, v) outb(v, p)
#define INPORT(p) inb(p)
#define SLEEP(t) usleep(t*1000) // microsseconds => milliseconds
#define LEN 150
#define MLEN 1
#define PlayNote(freq, time) { DoSound(freq, time); SLEEP(MLEN); }
#define LOBYTE(w) ((uchar) (w))
#define HIBYTE(w) ((uchar) (((uword) (w) >> 8) & 0xFF))

typedef unsigned short uword;
typedef unsigned char uchar;

  void HWSound(unsigned int freq) {
    uword counter = 1193280 / freq;    	
    OUTPORT(0x43, 0xB6);      		
    OUTPORT(0x42, LOBYTE(counter));    	
    OUTPORT(0x42, HIBYTE(counter));    	
    OUTPORT(0x61, INPORT(0x61) | 0x03); 
    return;
  }

  void HWNoSound(void) {
    OUTPORT(0x61, INPORT(0x61) & 0xFC); 
    return;
  }

  void HWDoSound(unsigned int freq, unsigned int time) {
    HWSound(freq);
    SLEEP(time);
    HWNoSound();
    return;
  }

  void DoSound(unsigned int freq, unsigned int time) {
    HWDoSound(freq, time);
    return;
  }
 
  static void shutup(int signal) {
      HWNoSound();
      exit(1);
  }

// Allarme di "file aggiunto" ***********************************************************
  void AlarmAddFile(void) {
    // * PERSONALIZZA ALLARME -----------------------*	
     PlayNote(523, LEN / 2);	  
     PlayNote(784, LEN / 2);    
     PlayNote(784, LEN / 2);    
     PlayNote(1046.50, LEN / 2);
    // *---------------------------------------------*
     return;
  } 

// Allarme di "file rimosso" ***********************************************************
  void AlarmSubFile(void) {
    // * PERSONALIZZA ALLARME -----------------------*
     PlayNote(130.81, LEN / 2);	  
     PlayNote(65.41, LEN);    
    // *---------------------------------------------*
     return;
  } 

  int CountFile(char * dirname) {
     DIR * dir; 
     struct dirent *direntry; 
     int ctf=-2; 
	
     if ((dir = opendir(dirname)) == NULL) {   
	 perror("spad error"); 
         return -1; 
     } 
	
     while ((direntry = readdir(dir)) != NULL) ctf++;
     closedir(dir);
     return ctf; 
  }
 
int main(int argc, char * argv[])
{
     int ctn, cto, xuid;
     unsigned int sec=0;
     char * dirname="";
     char * atype="";
	
     if (argc < 3) {
        printf("usage: %s directory pause [+/-]\n",argv[0]); 
	printf(" es: %s /home/user/Maildir/new 2 +\n\n",argv[0]);
	return -1; 
     }

     xuid = getuid();	

     if (xuid != 0) {
	fprintf(stderr, "Must be root.\n");
	return -1;
     }

     // "accende" lo speaker
    if ((ioperm(0x42, 2, 1) || ioperm(0x61, 1, 1) || ioperm(0x80, 1, 1))) {
	fprintf(stderr, "Error: speaker initializing.\n");
	return -1;
     }

     signal(SIGTERM, shutup);
     signal(SIGINT, shutup);
     signal(SIGHUP, shutup);

     sec=atoi(argv[2]);
     dirname=argv[1];
     if (argc == 4) atype=argv[3];

     cto=ctn=CountFile(dirname);
     if (ctn==-1) return -1;
   	
     for(;;) {
        sleep(sec);	
        ctn=CountFile(dirname);
        switch(*atype) {
	   case '+': if (ctn > cto) AlarmAddFile(); break;
           case '-': if (ctn < cto) AlarmSubFile(); break;
           default:
	      if (ctn > cto) AlarmAddFile();
  	      if (ctn < cto) AlarmSubFile();
        }	
        cto=ctn;
     }

     return 0; 
} 
